#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;
using System.Threading;
using System.Threading.Tasks;

namespace HeuristicLab.Optimization.Networks {
  [Item("ServicePort", "A service port of an optimization network node.")]
  [StorableClass]
  public class ServicePort : ClientServicePort, IServicePort {
    [StorableConstructor]
    protected ServicePort(bool deserializing) : base(deserializing) { }
    protected ServicePort(ServicePort original, Cloner cloner) : base(original, cloner) { }
    public ServicePort() : base("ServicePort") { }
    public ServicePort(string name) : base(name) { }
    public ServicePort(string name, string description) : base(name, description) { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ServicePort(this, cloner);
    }

    public ServiceParameterCollection Process(ServiceParameterCollection parameters) {
      return Process(parameters, CancellationToken.None);
    }
    public ServiceParameterCollection Process(ServiceParameterCollection parameters, CancellationToken token) {
      return OnProcessParameters(parameters, token);
    }
    public async Task<ServiceParameterCollection> ProcessAsync(ServiceParameterCollection parameters) {
      return await ProcessAsync(parameters, CancellationToken.None);
    }
    public async Task<ServiceParameterCollection> ProcessAsync(ServiceParameterCollection parameters, CancellationToken token) {
      return await Task.Run(() => { return Process(parameters, token); }, token);
    }

    public event EventHandler<EventArgs<ServiceParameterCollection, CancellationToken>> ProcessParameters;
    protected virtual ServiceParameterCollection OnProcessParameters(ServiceParameterCollection parameters, CancellationToken token) {
      var handler = ProcessParameters;
      if (handler != null) handler(this, new EventArgs<ServiceParameterCollection, CancellationToken>(parameters, token));
      return parameters;
    }
  }
}
